#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <syslog.h>

#include "outpost.h"
#include "common.h"

int _print (int dst, const char *format, ...)
{
  va_list args;
  
  va_start (args, format);

  if (dst) 
  {
    switch (dst)
    {
      case STYLE_COMMON:
        vfprintf (stdout, format, args);
      break;

      case STYLE_WARN:
        if (mode_daemon_e)
          vsyslog (SYSLOG_PRIORITY, format, args);
        else
          vfprintf (stdout, format, args);
      break;

      case STYLE_ALERT:
      case STYLE_EMERG:
        if (mode_daemon_e)
          vsyslog (SYSLOG_PRIORITY, format, args);
        else
          vfprintf (stderr, format, args);
      break;
    }
  }
  va_end (args);
  
  return (EXIT_SUCCESS);
} 

void *_malloc (size_t size)
{
  void *buf;

  if (!(buf = malloc (size)))
  {
    _print (STYLE_ALERT, "malloc failed: %s\n", sys_errlist[errno]);
    _exit (EXIT_FAILURE);
  }

  return (buf);
}

void *_realloc (void *ptr, size_t oldsize, size_t newsize)
{
  void *buf;

  buf = _malloc (newsize);
  memcpy (buf, ptr, (oldsize>newsize)?newsize:oldsize);
  _free (ptr);

  return (buf);
}

void _free (void *ptr)
{
  free (ptr);

  return;
}

char *_malloc_strcpy (char *str)
{
  char *buffer;
  int len;

  if (!str)
    return NULL;

  len = strlen (str);
  buffer = malloc (len + 1);
  memcpy (buffer, str, len + 1);

  return (buffer);
}

unsigned int _strlen (const char *str)
{
  char *b = (char *)str;

  if (!str)
    return (0);

  for (; *b; b++); 
  return (b - str);
}
