#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pcap.h>

#include "outpost.h"
#include "common.h"
#include "modules.h"
#include "config.h"

typedef enum 
{
  sBadOption,
  sLoadModule,
  sSnapLen,
  sModeDaemon,
  sBPFexpression,
  sInterface,
  sPidFile
} OpCodes;

configop keywords[] = {
  { "LoadModule", sLoadModule , 1},
  { "snaplen", sSnapLen, 1},
  { "modedaemon", sModeDaemon, 0 },
  { "bpf", sBPFexpression, 1 },
  { "interface", sInterface, 1 },
  { "pidfile", sPidFile, 1 },
  { NULL, 0 }
};

int checkunvalidity (const char *str)
{
  int i, len = _strlen (str);

  for (i = 0; i < len; i ++)
    if (*(str + i) == '#')
      return 1;

  return 0;
}

char *clean_string (const char *str)
{
  char *tmp, *savetmp, *final;
  int i, len;

  tmp = _malloc_strcpy ((char *)str);
  savetmp = tmp; 

  while (*tmp == ' ') tmp ++; 

  len = _strlen (tmp);

  for (i = 0; i < len; i ++)
    if (*(tmp + i) == '\n' || *(tmp + i) == '\r')
      *(tmp +i) = ' ';

  while (*(tmp + _strlen (tmp) - 1) == ' ') 
    *(tmp + _strlen (tmp) - 1) = 0;

  if (_strlen (tmp) == 0)
    final = NULL;
  else
    final = _malloc_strcpy (tmp);

  _free (savetmp);

  return (final);
}

int checkoption (const char *str, configop *list)
{
  char *confargtmp;
  configop *tmp = list;
  int i = 0;
  
  do
  {
    if (!strncasecmp(tmp->name, str, _strlen(tmp->name)))
    {
      confarg = NULL;
      if (tmp->arg)
      {
         /* la ligne requiert un argument, genre "LoadModule sniff"
          * On rcupre 'sniff' ici:
          */
        confargtmp = _malloc_strcpy ((char *)(str + _strlen (tmp->name)));
        confarg = clean_string (confargtmp);
        _free (confargtmp);

        if (!confarg)
          return (ESYNTAX);
      }
      return (tmp->opcode);
    }
    tmp = list + (++i); 
  }
  while (tmp->name != NULL);

  return (EUNKNOW);
}

int readconf ()
{
  FILE *conffile;
  char *opt;
  char *tmodule_name;
  char line[1024];
  int err = 0;

  /* XXX: handle more files */
  conffile = fopen ("./etc/outpost.conf", "r");

  if (conffile != NULL)
  {
    while (fgets (line, sizeof(line), conffile))
    {
      if (checkunvalidity (line))
        continue; 

      if (!(opt = clean_string (line)))
        continue;

      switch (checkoption(opt, keywords)) 
      {
        case EUNKNOW:
          _print (STYLE_ALERT, "(main) Config option '%s' unknown.\n", opt); 
          err = 1;
        break;

        case ESYNTAX:
          _print (STYLE_ALERT, 
            "(main) Config option '%s' requires an argument.\n", opt);
          err = 1;
        break;

        case sLoadModule: 
          tmodule_name = confarg;
          if (load_module_conf (conffile, tmodule_name))
          {
            _print(STYLE_ALERT, "(module) Problem loading module '%s' ?\n", 
              tmodule_name);
            err = 1;
          }
        break;

        case sSnapLen:
          snaplen = atoi (confarg);
        break;

        case sModeDaemon:
          mode_daemon = 1;
        break;

        case sBPFexpression:
          cmd = _malloc_strcpy(confarg);
        break;

        case sInterface:
          dev = _malloc_strcpy(confarg);
        break;

        case sPidFile: 
          pidfile = _malloc_strcpy(confarg);
        break;
      }
    if(err) break;
    }
    fclose (conffile);
  }
  else
    _print (STYLE_WARN, "not found configuration file.\n");

  return err;
}

int clearconf ()
{
  mode_test = 0;
  mode_real_test = 0;
  mode_daemon = 0; 
  mode_test_conf = 0;
  snaplen = 0;
  ignore_conf = 0;
  cmd = NULL;
  dev = NULL;
  mode_daemon_e = 0;
  layer2len = 0;

  return 0;
}

int initconf ()
{
  if (!snaplen) snaplen = 64;

  return 0;
}

