#include <unistd.h>
#include <stdlib.h>
#include <getopt.h>
#include <pcap.h>

//#include <netinet/in.h>
#ifndef __FAVOR_BSD
#define __FAVOR_BSD
#endif
#ifndef __USE_BSD
#define __USE_BSD
#endif
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>

#include "outpost.h"
#include "common.h"
#include "config.h"

/* XXX: for different ports, use reallocating 16/32/64 ...
 */

static struct option long_options[] =
{ 
  { "port", 1, 0, 'p' },
  { 0, 0, 0, 0 }
};

typedef enum
{
  sBadOption,
  sEndModule,
  sPort
} ModuleOpCodes;

configop mkeywords[] = {
  { "Port", sPort, 1 },
  { "EndModule", sEndModule, 0 },
  { "End", sEndModule, 0 },
  { NULL, 0 }
}; 

unsigned short *ports = NULL;
int nport = 0;

void addport (unsigned short port)
{
  if (ports)
  {
    if (!(nport % 16))
    {
      ports = _realloc (ports, sizeof(unsigned short) * nport, sizeof(unsigned short) * (nport + 16));
    }

    *(ports + nport) = port;
    nport ++;
  }
  else
  {
    ports = _malloc (sizeof(unsigned short) * 16);
    *ports = port;
    nport = 1;
  }
  return;
}

void listport ()
{
  int i;

  for (i = 0; i < nport; i ++) 
    _print (STYLE_COMMON, "==> %i\n", *(ports + i));

  return;
}

int module_load (int argc, char **argv, int *option_index, FILE *cf)
{
  int arg, fin = 0;
  char line [1024];
  int err = 0;
  char *opt;

  if (!cf)
  {
    do
    {
      arg = getopt_long (argc, argv, "p:e",
        long_options, option_index);

      switch (arg)
      {
        case 'p':
          _print (STYLE_WARN, "adding port: %i\n", atoi(optarg));
          addport (atoi(optarg));
        break;
        case 'e':
        default: fin = 1; break;
      }
    }
    while (arg != -1 && !fin);
  } // fin if (!cf)
  else
  {
    while (fgets (line, sizeof(line), cf) && !fin)
    {
      if (checkunvalidity (line))
        continue;
      if (!(opt = clean_string (line)))
        continue;

      switch (checkoption(opt, mkeywords))
      {
        case EUNKNOW:
          _print (STYLE_ALERT, "Config option '%s' unknown.\n", opt);
          err = 1;
        break;

        case ESYNTAX:
          _print (STYLE_ALERT, "Config option '%s' requires an argument.\n",
            opt);
          err = 1;
        break;

        case sEndModule:
          fin = 1;
        break;

        case sPort:
          addport (atoi (confarg));
        break;
      }
    }
    if (!fin)
    {
      _print (STYLE_ALERT, "Config file is prematured ended\n");
      err = 1;
    }
  }

  return err;
}

int module_free ()
{

  return 0;
}

int module_call (u_char *packet, struct pcap_pkthdr pcapstr)
{
  int i;
  struct ip *_iphdr;
  struct tcphdr *_tcphdr = NULL;
  struct udphdr *_udphdr = NULL;
  unsigned short port = 0;

  _iphdr = (struct ip*) (packet + layer2len);

  _print(STYLE_COMMON, "=>protocol: %i\n", _iphdr->ip_p); 

  if (_iphdr->ip_p == 6)
    _tcphdr = (struct tcphdr*) ((unsigned char*)_iphdr + (_iphdr->ip_hl<<2));

  if (_iphdr->ip_p == 17)
    _udphdr = (struct udphdr*) ((unsigned char*)_iphdr + (_iphdr->ip_hl<<2));

  if (!_tcphdr && !_udphdr)
    return 0;

  port = htons (_tcphdr->th_dport);

  for (i = 0; i < nport; i++)
  {
    if (port == *(ports + i))
    {
      /* alerte ! */
    }
  }
  return 0;
}
