/* gestion des modules
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dlfcn.h>
#include <pcap.h>

#include "common.h"
#include "modules.h"

plm modules = NULL;

plm load_module (char *name)
{
  char tmpmodule[1024];
  void *desc;
  plm tmpmods, newmod;
  struct stat tmpstat;

  sprintf (tmpmodule, "%s/module_%s.so", getcwd(NULL, 0), name);

  if (stat (tmpmodule, &tmpstat) == -1)
  {
    _print(STYLE_WARN, "[*] Not found module %s\n", name);
  }

  desc = dlopen (tmpmodule, RTLD_NOW);
  if (desc == NULL)
  {
    _print (STYLE_WARN, "[*] dlopen: %s\n", dlerror ());
    return (NULL);
  }

  tmpmods = modules;
  newmod = malloc (sizeof(struct loadedmodule_t));
  newmod->name = _malloc_strcpy (name);
  newmod->next = NULL;

  if (!(newmod->func_load = dlsym(desc, "module_load")))
    _print(STYLE_WARN, "module error: no module_load statement.\n");
  if (!(newmod->func_free = dlsym(desc, "module_free")))
    _print(STYLE_WARN, "module error: no module_free statement.\n");
  if (!(newmod->func_call = dlsym(desc, "module_call")))
    _print(STYLE_WARN, "module error: no module_call statement.\n");

  if (!modules)
  {
    modules = newmod;
  }
  else
  {
    for (; tmpmods->next; tmpmods = tmpmods->next);
    tmpmods->next = newmod;
  }

  _print (STYLE_COMMON, "Module %s loaded with success.\n", newmod->name);
  return (newmod);
}

int load_module_conf (FILE *cf, char *confarg)
{
  plm mod;
  if (!(mod = load_module (confarg)))
    return (1);

  return (mod->func_load (0, NULL, NULL, cf));
}

int load_module_arg (char *name, int argc, char **argv, int *index)
{
  plm mod;
  if (!(mod = load_module (name)))
    return (1);
  
  return (mod->func_load (argc, argv, index, NULL));
}

int unload_module_by_name (char *name)
{

  return (0);
}

int unload_all_module ()
{
  /* first module loaded, last module unloaded, this way */
  /* 1er module -> 2nd module -> ... -> last module */
  plm tmpmods, ltmpmods = NULL;

  if (!modules)
    return (0);

  do
  {
    for (tmpmods = modules; tmpmods->next; 
      ltmpmods = tmpmods, tmpmods = tmpmods->next);

/* Not that usefull ;-) */
/*    _print (1, "unloading module %s\n", tmpmods->name); */

    if (tmpmods == modules)
    {
      tmpmods->func_free ();
      free (tmpmods);
      tmpmods = NULL;
    }
    else
    {
      ltmpmods->next = NULL; 
      tmpmods->func_free ();
      free (tmpmods);
    }
  }
  while (tmpmods);

  return (0);
}

int list_modules ()
{
  plm tempmod;

  for (tempmod = modules; tempmod; tempmod = tempmod->next)
    _print (STYLE_COMMON, "module %s loaded\n", tempmod->name);

  return (0);
}

int module_do_action (const unsigned char *packet, struct pcap_pkthdr str)
{
  plm tempmod;

  for (tempmod = modules; tempmod; tempmod = tempmod->next)
  {
    tempmod->func_call (packet, str);
  }

  return (0);
}
